@@ -23,8 +23,12 @@ from .standard import ManufacturerData, ManufacturerDataField
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_BLE.git"
 
-_MANUFACTURING_DATA_ADT = const(0xFF)
-_ADAFRUIT_COMPANY_ID = const(0x0822)
+MANUFACTURING_DATA_ADT = const(0xFF)
+"""The advertising data type for manufacturer-specific data"""
+
+ADAFRUIT_COMPANY_ID = const(0x0822)
+"""Company Identifier for Adafruit Industries"""
+
 _COLOR_DATA_ID = const(0x0000)
 
 
@@ -35,8 +39,8 @@ class AdafruitColor(Advertisement):
     match_prefixes = (
         struct.pack(
             "<BHBH",
-            _MANUFACTURING_DATA_ADT,
-            _ADAFRUIT_COMPANY_ID,
+            MANUFACTURING_DATA_ADT,
+            ADAFRUIT_COMPANY_ID,
             struct.calcsize("<HI"),
             _COLOR_DATA_ID,
         ),
@@ -44,8 +48,8 @@ class AdafruitColor(Advertisement):
     manufacturer_data = LazyObjectField(
         ManufacturerData,
         "manufacturer_data",
-        advertising_data_type=_MANUFACTURING_DATA_ADT,
-        company_id=_ADAFRUIT_COMPANY_ID,
+        advertising_data_type=MANUFACTURING_DATA_ADT,
+        company_id=ADAFRUIT_COMPANY_ID,
         key_encoding="<H",
     )
     color = ManufacturerDataField(_COLOR_DATA_ID, "<I")
