@@ -10,9 +10,22 @@ This module provides integer characteristics that are usable directly as attribu
 
 """
 
+from __future__ import annotations
+
 from . import Attribute
 from . import StructCharacteristic
 
+try:
+    from typing import Optional, Type, Union, TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        from circuitpython_typing import ReadableBuffer
+        from adafruit_ble.uuid import UUID
+        from adafruit_ble.services import Service
+
+except ImportError:
+    pass
+
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_BLE.git"
 
@@ -22,16 +35,16 @@ class IntCharacteristic(StructCharacteristic):
 
     def __init__(
         self,
-        format_string,
-        min_value,
-        max_value,
+        format_string: str,
+        min_value: int,
+        max_value: int,
         *,
-        uuid=None,
-        properties=0,
-        read_perm=Attribute.OPEN,
-        write_perm=Attribute.OPEN,
-        initial_value=None
-    ):
+        uuid: Optional[UUID] = None,
+        properties: int = 0,
+        read_perm: int = Attribute.OPEN,
+        write_perm: int = Attribute.OPEN,
+        initial_value: Optional[ReadableBuffer] = None,
+    ) -> None:
         self._min_value = min_value
         self._max_value = max_value
         if initial_value is not None:
@@ -48,12 +61,14 @@ class IntCharacteristic(StructCharacteristic):
             initial_value=initial_value,
         )
 
-    def __get__(self, obj, cls=None):
+    def __get__(
+        self, obj: Optional[Service], cls: Optional[Type[Service]] = None
+    ) -> Union[int, "IntCharacteristic"]:
         if obj is None:
             return self
         return super().__get__(obj)[0]
 
-    def __set__(self, obj, value):
+    def __set__(self, obj: Service, value: int) -> None:
         if not self._min_value <= value <= self._max_value:
             raise ValueError("out of range")
         super().__set__(obj, (value,))
@@ -63,7 +78,9 @@ class Int8Characteristic(IntCharacteristic):
     """Int8 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=-128, max_value=127, **kwargs):
+    def __init__(
+        self, *, min_value: int = -128, max_value: int = 127, **kwargs
+    ) -> None:
         super().__init__("<b", min_value, max_value, **kwargs)
 
 
@@ -71,7 +88,7 @@ class Uint8Characteristic(IntCharacteristic):
     """Uint8 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=0, max_value=0xFF, **kwargs):
+    def __init__(self, *, min_value: int = 0, max_value: int = 0xFF, **kwargs) -> None:
         super().__init__("<B", min_value, max_value, **kwargs)
 
 
@@ -79,7 +96,9 @@ class Int16Characteristic(IntCharacteristic):
     """Int16 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=-32768, max_value=32767, **kwargs):
+    def __init__(
+        self, *, min_value: int = -32768, max_value: int = 32767, **kwargs
+    ) -> None:
         super().__init__("<h", min_value, max_value, **kwargs)
 
 
@@ -87,7 +106,9 @@ class Uint16Characteristic(IntCharacteristic):
     """Uint16 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=0, max_value=0xFFFF, **kwargs):
+    def __init__(
+        self, *, min_value: int = 0, max_value: int = 0xFFFF, **kwargs
+    ) -> None:
         super().__init__("<H", min_value, max_value, **kwargs)
 
 
@@ -95,7 +116,9 @@ class Int32Characteristic(IntCharacteristic):
     """Int32 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=-2147483648, max_value=2147483647, **kwargs):
+    def __init__(
+        self, *, min_value: int = -2147483648, max_value: int = 2147483647, **kwargs
+    ) -> None:
         super().__init__("<i", min_value, max_value, **kwargs)
 
 
@@ -103,5 +126,7 @@ class Uint32Characteristic(IntCharacteristic):
     """Uint32 number."""
 
     # pylint: disable=too-few-public-methods
-    def __init__(self, *, min_value=0, max_value=0xFFFFFFFF, **kwargs):
+    def __init__(
+        self, *, min_value: int = 0, max_value: int = 0xFFFFFFFF, **kwargs
+    ) -> None:
         super().__init__("<I", min_value, max_value, **kwargs)
