@@ -10,10 +10,23 @@ This module provides a JSON characteristic for reading/writing JSON serializable
 
 """
 
+from __future__ import annotations
+
 import json
 from . import Attribute
 from . import Characteristic
 
+try:
+    from typing import Optional, Any, Type, TYPE_CHECKING
+
+    if TYPE_CHECKING:
+        from circuitpython_typing import ReadableBuffer
+        from adafruit_ble.uuid import UUID
+        from adafruit_ble.services import Service
+
+except ImportError:
+    pass
+
 __version__ = "0.0.0-auto.0"
 __repo__ = "https://github.com/adafruit/Adafruit_CircuitPython_BLE.git"
 
@@ -24,12 +37,12 @@ class JSONCharacteristic(Characteristic):
     def __init__(
         self,
         *,
-        uuid=None,
-        properties=Characteristic.READ,
-        read_perm=Attribute.OPEN,
-        write_perm=Attribute.OPEN,
-        initial_value=None,
-    ):
+        uuid: Optional[UUID] = None,
+        properties: int = Characteristic.READ,
+        read_perm: int = Attribute.OPEN,
+        write_perm: int = Attribute.OPEN,
+        initial_value: Optional[ReadableBuffer] = None,
+    ) -> None:
         super().__init__(
             uuid=uuid,
             properties=properties,
@@ -41,19 +54,21 @@ class JSONCharacteristic(Characteristic):
         )
 
     @staticmethod
-    def pack(value):
+    def pack(value: Any) -> bytes:
         """Converts a JSON serializable python value into a utf-8 encoded JSON string."""
         return json.dumps(value).encode("utf-8")
 
     @staticmethod
-    def unpack(value):
+    def unpack(value: ReadableBuffer) -> Any:
         """Converts a utf-8 encoded JSON string into a python value."""
         return json.loads(str(value, "utf-8"))
 
-    def __get__(self, obj, cls=None):
+    def __get__(
+        self, obj: Optional[Service], cls: Optional[Type[Service]] = None
+    ) -> Any:
         if obj is None:
             return self
         return self.unpack(super().__get__(obj, cls))
 
-    def __set__(self, obj, value):
+    def __set__(self, obj: Service, value: Any) -> None:
         super().__set__(obj, self.pack(value))
